@@ -60,4 +60,4 @@ from .abstract_classes import *
 from .parse_utils import (num_tokens_from_string,
                           chunk_source_code,
                           chunk_any_to_tokens)
-from .log_utils import get_logFile
+from .log_utils import get_logFile,AbstractLogManager
