@@ -24,7 +24,7 @@ Version: 0.1.2
 """
 import os
 import platform
-from pathlib import Path
+
 def get_os_info():
     """
     Get Operating System Information
@@ -301,14 +301,14 @@ def mkGbTrunFroPathTot(k) -> float:
     return trunc(mkGb(s.path.getsize(k)), 5)
 
 
-def get_abs_name_of_this() -> Path:
+def get_abs_name_of_this():
     """
     Returns the absolute name of the current module.
 
     Returns:
         Path: The absolute name of the current module.
     """
-    return Path(__name__).absolute()
+    return os.path.abspath(__name__)
 
 def createFolds(ls: list) -> None:
     """
