@@ -1,45 +1,63 @@
 """
 type_utils.py
-This module provides utility functions for working with data types.
+
+This module provides a collection of utility functions for type checking and conversion.
+It includes functions to determine the type of an object, check if an object is a specific type,
+and perform type conversions. These functions help simplify the process of handling different
+types of data and ensure consistent behavior across different data types.
+
+Usage:
+    import abstract_utilities.type_utils as type_utils
 
 Functions:
-- is_number(obj) -> bool: Checks if the input object is a number.
-- is_str(obj) -> bool: Checks if the input object is a string.
-- is_int(obj) -> bool: Checks if the input object is an integer.
-- get_type(obj) -> any: Determines the type of the input object.
-- is_float(obj) -> bool: Checks if the input object is a float.
-- is_bool(obj) -> bool: Checks if the input object is a boolean.
-- is_list(obj) -> bool: Checks if the input object is a list.
-- is_tuple(obj) -> bool: Checks if the input object is a tuple.
-- is_set(obj) -> bool: Checks if the input object is a set.
-- is_dict(obj) -> bool: Checks if the input object is a dictionary.
-- is_frozenset(obj) -> bool: Checks if the input object is a frozenset.
-- is_bytearray(obj) -> bool: Checks if the input object is a bytearray.
-- is_bytes(obj) -> bool: Checks if the input object is a bytes object.
-- is_memoryview(obj) -> bool: Checks if the input object is a memoryview object.
-- is_range(obj) -> bool: Checks if the input object is a range object.
-- is_enumerate(obj) -> bool: Checks if the input object is an enumerate object.
-- is_zip(obj) -> bool: Checks if the input object is a zip object.
-- is_filter(obj) -> bool: Checks if the input object is a filter object.
-- is_map(obj) -> bool: Checks if the input object is a map object.
-- is_property(obj) -> bool: Checks if the input object is a property object.
-- is_slice(obj) -> bool: Checks if the input object is a slice object.
-- is_super(obj) -> bool: Checks if the input object is a super object.
-- is_type(obj) -> bool: Checks if the input object is a type object.
-- is_Exception(obj) -> bool: Checks if the input object is an Exception object.
-- is_none(obj) -> bool: Checks if the input object is None.
-- is_dict_or_convertable(obj) -> bool: Checks if the input object is a dictionary or can be converted to a dictionary.
-- dict_check_conversion(obj) -> any: Converts the input object to a dictionary if possible.
-- make_list(obj) -> list: Converts the input object to a list.
-- make_list_lower(ls: list) -> list: Converts all elements in a list to lowercase, ignoring None values.
-- make_float(x) -> float: Converts the input object to a float.
-- make_bool(x) -> bool: Converts the input object to a boolean.
-- make_str(x: any) -> str: Converts the input object to a string.
-- get_obj_obj(obj: str, x: any) -> any: Returns the object converted according to the given type string.
-- get_len_or_num(obj: any) -> int: Returns the length of the object if it can be converted to a string, else the integer representation of the object.
-- get_types_list() -> list: Returns a list of all supported types.
+- is_iterable(obj: any) -> bool
+- is_number(obj: any) -> bool
+- is_str(obj: any) -> bool
+- is_int(obj: any) -> bool
+- is_float(obj: any) -> bool
+- is_bool(obj: any) -> bool
+- is_list(obj: any) -> bool
+- is_tuple(obj: any) -> bool
+- is_set(obj: any) -> bool
+- is_dict(obj: any) -> bool
+- is_frozenset(obj: any) -> bool
+- is_bytearray(obj: any) -> bool
+- is_bytes(obj: any) -> bool
+- is_memoryview(obj: any) -> bool
+- is_range(obj: any) -> bool
+- is_enumerate(obj: any) -> bool
+- is_zip(obj: any) -> bool
+- is_filter(obj: any) -> bool
+- is_map(obj: any) -> bool
+- is_property(obj: any) -> bool
+- is_slice(obj: any) -> bool
+- is_super(obj: any) -> bool
+- is_type(obj: any) -> bool
+- is_Exception(obj: any) -> bool
+- is_none(obj: any) -> bool
+- is_str_convertible_dict(obj: any) -> bool
+- is_dict_or_convertable(obj: any) -> bool
+- dict_check_conversion(obj: any) -> Union[dict, any]
+- make_list(obj: any) -> list
+- make_list_lower(ls: list) -> list
+- make_float(obj: Union[str, float, int]) -> float
+- make_bool(obj: Union[bool, int, str]) -> Union[bool, str]
+- make_str(obj: any) -> str
+- get_obj_obj(obj_type: str, obj: any) -> any
+- get_len_or_num(obj: any) -> int
+- get_types_list() -> list
+- det_bool_F(obj: (tuple or list or bool) = False) -> bool
+- det_bool_T(obj: (tuple or list or bool) = False) -> bool
+- T_or_F_obj_eq(event: any = '', obj: any = '') -> bool
+
+This module is part of the `abstract_utilities` package.
+
+Author: putkoff
+Date: 05/31/2023
+Version: 0.1.2
 """
-def is_iterable(obj):
+from typing import Union
+def is_iterable(obj:any):
     try:
         iterator=iter(obj)
     except TypeError:
@@ -47,49 +65,8 @@ def is_iterable(obj):
     else:
         return True
     return True
-def is_number(obj) -> bool:
-    """
-    Checks if the input object is a number.
-    
-    Args:
-        obj: The object to check.
-        
-    Returns:
-        bool: True if the object is a number (int or float), False otherwise.
-    """
-    try:
-        float(obj)
-        return True
-    except ValueError:
-        return False
 
-
-def is_str(obj) -> bool:
-    """
-    Checks if the input object is a string.
-    
-    Args:
-        obj: The object to check.
-        
-    Returns:
-        bool: True if the object is a string, False otherwise.
-    """
-    return isinstance(obj, str)
-
-
-def is_int(obj) -> bool:
-    """
-    Checks if the input object is an integer.
-    
-    Args:
-        obj: The object to check.
-        
-    Returns:
-        bool: True if the object is an integer, False otherwise.
-    """
-    return isinstance(obj, int)
-
-def get_type(obj) -> any:
+def get_type(obj:any) -> any:
     """
     Determines the type of the input object.
 
@@ -108,28 +85,8 @@ def get_type(obj) -> any:
     elif is_str(obj):
         obj = str(obj)
     return obj
-def is_float(obj) -> bool:
-    """
-    Checks if the input object is a float.
-    
-    Args:
-        obj: The object to check.
-        
-    Returns:
-        bool: True if the object is a float, False otherwise.
-    """
-    return isinstance(obj, float)
-def get_type(obj):
-    if is_number(obj):
-        obj = int(obj)
-    if is_float(obj):
-        return float(obj)
-    elif obj == 'None':
-        obj =  None
-    elif is_str(obj):
-        obj =  str(obj)
-    return obj
-def is_number(obj) -> bool:
+
+def is_number(obj:any) -> bool:
     """
     Checks whether the input object can be represented as a number.
 
@@ -144,7 +101,7 @@ def is_number(obj) -> bool:
         return True
     except (TypeError, ValueError):
         return False
-def is_object(obj) -> bool:
+def is_object(obj:any) -> bool:
     """
     Checks whether the input object is of type 'object'.
 
@@ -155,7 +112,7 @@ def is_object(obj) -> bool:
         bool: True if the object is of type 'object', False otherwise.
     """
     return isinstance(obj, object)
-def is_str(obj) -> bool:
+def is_str(obj:any) -> bool:
     """
     Checks whether the input object is of type 'str'.
 
@@ -166,7 +123,7 @@ def is_str(obj) -> bool:
         bool: True if the object is of type 'str', False otherwise.
     """
     return isinstance(obj, str)
-def is_int(obj) -> bool:
+def is_int(obj:any) -> bool:
     """
     Checks whether the input object is of type 'int'.
 
@@ -177,7 +134,7 @@ def is_int(obj) -> bool:
         bool: True if the object is of type 'int', False otherwise.
     """
     return isinstance(obj, int)
-def is_float(obj) -> bool:
+def is_float(obj:any) -> bool:
     """
     Checks whether the input object is of type 'float'.
 
@@ -188,7 +145,7 @@ def is_float(obj) -> bool:
         bool: True if the object is of type 'float', False otherwise.
     """
     return isinstance(obj, float)
-def is_bool(obj) -> bool:
+def is_bool(obj:any) -> bool:
     """
     Checks whether the input object is of type 'bool'.
 
@@ -201,7 +158,7 @@ def is_bool(obj) -> bool:
     return isinstance(obj, bool)
 
 
-def is_list(obj) -> bool:
+def is_list(obj:any) -> bool:
     """
     Checks whether the input object is of type 'list'.
 
@@ -212,7 +169,7 @@ def is_list(obj) -> bool:
         bool: True if the object is of type 'list', False otherwise.
     """
     return isinstance(obj, list)
-def is_tuple(obj) -> bool:
+def is_tuple(obj:any) -> bool:
     """
     Checks whether the input object is of type 'tuple'.
 
@@ -223,7 +180,7 @@ def is_tuple(obj) -> bool:
         bool: True if the object is of type 'tuple', False otherwise.
     """
     return isinstance(obj, tuple)
-def is_set(obj) -> bool:
+def is_set(obj:any) -> bool:
     """
     Checks whether the input object is of type 'set'.
 
@@ -234,7 +191,7 @@ def is_set(obj) -> bool:
         bool: True if the object is of type 'set', False otherwise.
     """
     return isinstance(obj, set)
-def is_dict(obj) -> bool:
+def is_dict(obj:any) -> bool:
     """
     Checks whether the input object is of type 'dict'.
 
@@ -245,7 +202,7 @@ def is_dict(obj) -> bool:
         bool: True if the object is of type 'dict', False otherwise.
     """
     return isinstance(obj, dict)
-def is_frozenset(obj) -> bool:
+def is_frozenset(obj:any) -> bool:
     """
     Checks whether the input object is of type 'frozenset'.
 
@@ -256,7 +213,7 @@ def is_frozenset(obj) -> bool:
         bool: True if the object is of type 'frozenset', False otherwise.
     """
     return isinstance(obj, frozenset)
-def is_bytearray(obj) -> bool:
+def is_bytearray(obj:any) -> bool:
     """
     Checks whether the input object is of type 'bytearray'.
 
@@ -267,7 +224,7 @@ def is_bytearray(obj) -> bool:
         bool: True if the object is of type 'bytearray', False otherwise.
     """
     return isinstance(obj, bytearray)
-def is_bytes(obj) -> bool:
+def is_bytes(obj:any) -> bool:
     """
     Checks whether the input object is of type 'bytes'.
 
@@ -278,7 +235,7 @@ def is_bytes(obj) -> bool:
         bool: True if the object is of type 'bytes', False otherwise.
     """
     return isinstance(obj, bytes)
-def is_memoryview(obj) -> bool:
+def is_memoryview(obj:any) -> bool:
     """
     Checks whether the input object is of type 'memoryview'.
 
@@ -289,7 +246,7 @@ def is_memoryview(obj) -> bool:
         bool: True if the object is of type 'memoryview', False otherwise.
     """
     return isinstance(obj, memoryview)
-def is_range(obj) -> bool:
+def is_range(obj:any) -> bool:
     """
     Checks whether the input object is
 
@@ -302,7 +259,7 @@ def is_range(obj) -> bool:
         bool: True if the object is of type 'range', False otherwise.
     """
     return isinstance(obj, range)
-def is_enumerate(obj) -> bool:
+def is_enumerate(obj:any) -> bool:
     """
     Checks whether the input object is of type 'enumerate'.
 
@@ -313,7 +270,7 @@ def is_enumerate(obj) -> bool:
         bool: True if the object is of type 'enumerate', False otherwise.
     """
     return isinstance(obj, enumerate)
-def is_zip(obj) -> bool:
+def is_zip(obj:any) -> bool:
     """
     Checks whether the input object is of type 'zip'.
 
@@ -324,7 +281,7 @@ def is_zip(obj) -> bool:
         bool: True if the object is of type 'zip', False otherwise.
     """
     return isinstance(obj, zip)
-def is_filter(obj) -> bool:
+def is_filter(obj:any) -> bool:
     """
     Checks whether the input object is of type 'filter'.
 
@@ -335,7 +292,7 @@ def is_filter(obj) -> bool:
         bool: True if the object is of type 'filter', False otherwise.
     """
     return isinstance(obj, filter)
-def is_map(obj) -> bool:
+def is_map(obj:any) -> bool:
     """
     Checks whether the input object is of type 'map'.
 
@@ -346,7 +303,7 @@ def is_map(obj) -> bool:
         bool: True if the object is of type 'map', False otherwise.
     """
     return isinstance(obj, map)
-def is_property(obj) -> bool:
+def is_property(obj:any) -> bool:
     """
     Checks whether the input object is of type 'property'.
 
@@ -359,7 +316,7 @@ def is_property(obj) -> bool:
     return isinstance(obj, property)
 
 
-def is_slice(obj) -> bool:
+def is_slice(obj:any) -> bool:
     """
     Checks whether the input object is of type 'slice'.
 
@@ -372,7 +329,7 @@ def is_slice(obj) -> bool:
     return isinstance(obj, slice)
 
 
-def is_super(obj) -> bool:
+def is_super(obj:any) -> bool:
     """
     Checks whether the input object is of type 'super'.
 
@@ -385,7 +342,7 @@ def is_super(obj) -> bool:
     return isinstance(obj, super)
 
 
-def is_type(obj) -> bool:
+def is_type(obj:any) -> bool:
     """
     Checks whether the input object is of type 'type'.
 
@@ -398,7 +355,7 @@ def is_type(obj) -> bool:
     return isinstance(obj, type)
 
 
-def is_Exception(obj) -> bool:
+def is_Exception(obj:any) -> bool:
     """
     Checks whether the input object is of type 'Exception'.
 
@@ -411,7 +368,7 @@ def is_Exception(obj) -> bool:
     return isinstance(obj, Exception)
 
 
-def is_none(obj) -> bool:
+def is_none(obj:any) -> bool:
     """
     Checks whether the input object is of type 'None'.
 
@@ -427,43 +384,8 @@ def is_none(obj) -> bool:
         return False
 
 
-def is_dict(obj) -> bool:
-    """
-    Checks whether the input object is of type 'dict'.
 
-    Args:
-        obj: The object to check.
-
-    Returns:
-        bool: True if the object is of type 'dict', False otherwise.
-    """
-    if isinstance(obj, dict):
-        return True
-    return False
-
-
-def is_str_convertible_dict(obj) -> bool:
-    """
-    Checks whether the input object can be converted to a dictionary.
-
-    Args:
-        obj
-
-: The object to check.
-
-    Returns:
-        bool: True if the object can be converted to a dictionary, False otherwise.
-    """
-    if isinstance(obj, str):
-        try:
-            obj = json.loads(obj)
-            return True
-        except json.JSONDecodeError:
-            return False
-    return False
-
-
-def is_dict_or_convertable(obj) -> bool:
+def is_dict_or_convertable(obj:any) -> bool:
     """
     Checks whether the input object is of type 'dict' or can be converted to a dictionary.
 
@@ -478,39 +400,7 @@ def is_dict_or_convertable(obj) -> bool:
     if is_str_convertible_dict(obj):
         return True
     return False
-
-
-def dict_check_conversion(obj) -> any:
-    """
-    Converts the input object to a dictionary if possible.
-
-    Args:
-        obj: The object to convert.
-
-    Returns:
-        any: The converted object if conversion is possible, otherwise the original object.
-    """
-    if is_dict_or_convertable(obj):
-        if is_dict(obj):
-            return obj
-        return json.loads(obj)
-    return obj
-
-
-def is_list(obj) -> bool:
-    """
-    Checks whether the input object is of type 'list'.
-
-    Args:
-        obj: The object to check.
-
-    Returns:
-        bool: True if the object is of type 'list', False otherwise.
-    """
-    if type(obj) is list:
-        return True
-    return False
-def is_str_convertible_dict(obj) -> bool:
+def is_str_convertible_dict(obj:any) -> bool:
     """
     Checks whether the input object is a string that can be converted to a dict.
 
@@ -531,19 +421,7 @@ def is_str_convertible_dict(obj) -> bool:
 
     return False
 
-def is_dict_or_convertable(obj) -> bool:
-    """
-    Checks whether the input object is a dictionary or can be converted to a dictionary.
-
-    Args:
-        obj: The object to check.
-
-    Returns:
-        bool: True if the object is a dictionary or can be converted to one, False otherwise.
-    """
-    return is_dict(obj) or is_str_convertible_dict(obj)
-
-def dict_check_conversion(obj):
+def dict_check_conversion(obj:any) -> Union[dict,any]:
     """
     Converts the input object to a dictionary if possible.
 
@@ -561,7 +439,7 @@ def dict_check_conversion(obj):
         return json.loads(obj)
     
     return obj
-def make_list(obj) -> list:
+def make_list(obj:any) -> list:
     """
     Converts the input object to a list. If the object is already a list, it is returned as is.
     
@@ -571,7 +449,10 @@ def make_list(obj) -> list:
     Returns:
         list: The object as a list.
     """
-    return obj if isinstance(obj, list) else [obj]
+    if isinstance(obj, list):
+        return obj
+    return [obj]
+    
 def make_list_lower(ls: list) -> list:
     """
     Converts all elements in a list to lowercase. Ignores None values.
@@ -585,7 +466,7 @@ def make_list_lower(ls: list) -> list:
     return [item.lower() if isinstance(item, str) else item for item in ls]
 
 
-def make_float(x) -> float:
+def make_float(obj:Union[str,float,int]) -> float:
     """
     Converts the input object to a float.
     
@@ -596,82 +477,80 @@ def make_float(x) -> float:
         float: The float representation of the object.
     """
     try:
-        return float(x)
+        return float(obj)
     except (TypeError, ValueError):
         return 1.0
 
+def make_bool(obj: Union[bool, int, str]) -> Union[bool, str]:
+    """
+    Converts the input object to a boolean representation if possible.
 
-def make_bool(x) -> bool:
-    """
-    Converts the input object to a boolean.
-    
-    Args:
-        x: The object to convert.
-        
-    Returns:
-        bool: The boolean representation of the object. Returns None if the conversion is not possible.
-    """
-def make_bool(obj):
-    """
-    Converts the input object to a boolean.
+    The function attempts to convert various objects, including integers and strings, to their boolean equivalents. 
+    If the conversion is not possible, the original object is returned.
 
     Args:
-        x: The object to convert.
+        obj: The object to be converted.
 
     Returns:
-        bool: The boolean representation of the object. Returns None if the conversion is not possible.
+        bool or original type: The boolean representation of the object if conversion is possible. Otherwise, it returns the original object.
+
+    Examples:
+        make_bool("true") -> True
+        make_bool(1)      -> True
+        make_bool("0")    -> False
+        make_bool(2)      -> 2
     """
-    if isinstance(obj,bool):
+    if isinstance(obj, bool):
         return obj
-    if isinstance(obj,int):
-       if obj == 0:
-           return False
-        if obj == 1:
-           return True
-    if isinstance(obj,str):
-        if obj.lower() in ['0',"false"]:
+    if isinstance(obj, int):
+        if obj == 0:
             return False
-        if obj.lower() in ['1',"true"]:
+        if obj == 1:
+            return True
+    if isinstance(obj, str):
+        if obj.lower() in ['0', "false"]:
+            return False
+        if obj.lower() in ['1', "true"]:
             return True
     return obj
 
-def make_str(x: any) -> str:
+def make_str(obj: any) -> str:
     """
     Converts the input object to a string.
     
     Args:
-        x: The object to convert.
+        obj: The object to convert.
         
     Returns:
         str: The string representation of the object.
     """
-    return str(x)
+    return str(obj)
 
 
-def get_obj_obj(obj: str, x: any) -> any:
+def get_obj_obj(obj_type: str, obj: any) -> any:
     """
     Returns the object converted according to the given type string.
     
     Args:
-        obj: The string representing the type to convert to.
-        x: The object to convert.
+        obj_type: The string representing the type to convert to.
+        obj: The object to convert.
         
     Returns:
         any: The object converted to the specified type.
     """
-    if obj == 'str':
-        return make_str(x)
-    elif obj == 'bool':
-        return make_bool(x)
-    elif obj == 'float':
-        return make_float(x)
-    elif obj == 'int':
+    if obj_type == 'str':
+        return make_str(obj)
+    elif obj_type == 'bool':
+        return make_bool(obj)
+    elif obj_type == 'float':
+        return make_float(obj)
+    elif obj_type == 'int':
         try:
-            return int(x)
+            return int(obj)
         except (TypeError, ValueError):
-            return x
+            return obj
     else:
-        return x
+        return obj
 def get_len_or_num(obj: any) -> int:
     """
     Returns the length of the object if it can be converted to a string, else the integer representation of the object.
@@ -691,6 +570,117 @@ def get_len_or_num(obj: any) -> int:
             return 0
 def get_types_list()->list:
     return ['list', 'bool', 'str', 'int', 'float', 'set', 'dict', 'frozenset', 'bytearray', 'bytes', 'memoryview', 'range', 'enumerate', 'zip', 'filter', 'map', 'property', 'slice', 'super', 'type', 'Exception', 'NoneType']
+def det_bool_F(obj: (tuple or list or bool) = False):
+    """
+    Determines if the given object is a boolean False value.
+
+    Args:
+        obj (tuple or list or bool): The object to determine the boolean False value.
+
+    Returns:
+        bool: True if the object is a boolean False value, False otherwise.
+    """
+    if isinstance(obj, bool):
+        return obj
+    return all(obj)
+def det_bool_T(obj: (tuple or list or bool) = False):
+    """
+    Determines if the given object is a boolean True value.
+
+    Args:
+        obj (tuple or list or bool): The object to determine the boolean True value.
+
+    Returns:
+        bool: True if the object is a boolean True value, False otherwise.
+    """
+    if isinstance(obj, bool):
+        return obj 
+    return any(obj)
+def T_or_F_obj_eq(event: any = '', obj: any = ''):
+    """
+    Compares two objects and returns True if they are equal, False otherwise.
+
+    Args:
+        event (any): The first object to compare.
+        obj (any): The second object to compare.
+
+    Returns:
+        bool: True if the objects are equal, False otherwise.
+    """
+    return True if event == obj else False
+def ensure_integer(page_value:any, default_value:int):
+    """
+    Ensures the given value is an integer. If not, it tries to extract 
+    the numeric part of the value. If still unsuccessful, it defaults 
+    to the given default value.
+
+    Parameters:
+    - page_value (str|int|any): The value to ensure as integer. 
+                                Non-numeric characters are stripped if necessary.
+    - default_value (int): The default value to return if conversion 
+                           to integer is unsuccessful.
+
+    Returns:
+    - int: The ensured integer value.
+    """
+    # Check if page_value is already a number
+    if not is_number(page_value):
+        # Convert to string in case it's not already
+        page_value = str(page_value)
+        
+        # Remove non-numeric characters from the beginning
+        while len(page_value) > 0 and page_value[0] not in '0123456789'.split(','):
+            page_value = page_value[1:]
+
+        # Remove non-numeric characters from the end
+        while len(page_value) > 0 and page_value[-1] not in '0123456789'.split(','):
+            page_value = page_value[:-1]
+
+    # If page_value is empty or still not a number, use the default value
+    if len(page_value) == 0 or not is_number(page_value):
+        return default_value
+
+    # Convert page_value to an integer and return
+    return int(page_value)
+def if_default_return_obj(obj:any,default:any=None,default_compare:any=None):
+    if default == default_compare:
+        return obj
+    return default
+
+def convert_and_normalize_values(values):
+    for value in values:
+        if isinstance(value, str):
+            yield value.lower()
+        elif isinstance(value, (int, float)):
+            yield value
+        else:
+            yield str(value).lower()
+            
+def convert_to_number(value):
+    value_str = str(value)
+    if is_number(value_str):
+        return float(value_str) if '.' in value_str else int(value_str)
+    return value_str
+def is_number(s):
+    if s:
+        try:
+            float(s)
+            return True
+        except:
+            return False
+    return False
+
+def makeInt(obj):
+    if is_number(obj):
+       return int(obj)
+    return obj
+
+def str_lower(obj):
+    try:
+        obj=str(obj).lower()
+    except Exception as e:
+        print(f"{e}")
+    return obj
 
 # Function: is_number
 # Function: is_str
